# common initialization and support classes for zwiki unit tests

import string, re, os, sys

import unittest
# support older pyunit/python/zope
if not hasattr(unittest.TestCase,'assertEquals'):
    def assertEquals(self,first,second, msg=None):
        if first != second:
            raise AssertionError, (msg or '%s != %s' % (first, second))
    unittest.TestCase.assertEqual = \
        unittest.TestCase.assertEquals = assertEquals

# ensure "Products.ZWiki" will work in:
# normal zope, testrunner, direct invocation, python1.5, python2
# import App.FindHomes doesn't work because testrunner changes INSTANCE_HOME
import os
def pdir(path): return os.path.split(path)[0]
thisProductsDir = pdir(pdir(pdir(os.path.abspath(__file__))))
import Products
Products.__path__.insert(0,thisProductsDir)

# always needs to be imported early to set up Persistence.Persistent
import ZODB

# mock objects
import OFS.ObjectManager, AccessControl.User, DateTime
from ZPublisher.HTTPRequest import HTTPRequest
from ZPublisher.HTTPResponse import HTTPResponse
from Products import ZWiki
from Products.ZWiki.ZWikiPage import ZWikiPage, ZOPEVERSION
from Products.ZWiki.Mail import MailSupport

class MockUser:
    def __init__(self,username='testuser'):
        self.username = username
        class aclusers: pass
        self.acl_users = aclusers()
        self.acl_users._nobody = 'nobody'
        
    def getUserName(self):
        return self.username


class MockRequest(HTTPRequest):
    """
    a mock HTTPRequest object for use in testing.

    like makerequest without the app dependency
    """
    def __init__(self):
        resp = HTTPResponse(stdout=sys.stdout)
        environ={}
        environ['SERVER_NAME']='foo'
        environ['SERVER_PORT']='80'
        environ['REQUEST_METHOD'] = 'GET'
        environ['SCRIPT_NAME']='/foo/test'
        HTTPRequest.__init__(self,None,environ,resp)


class MockZWikiPage(ZWikiPage):
    """
    a mock ZWikiPage object for use in testing.

    We fake acquisition below, good enough for some tests; to get the real
    thing, use p = MockZWikiPage().aq_parent.TestPage .

    XXX problem: page methods like standard_wiki_header appear as
    attributes of the parent folder, breaking tests
    """
    def __init__(self, source_string='', mapping=None, __name__='TestPage',
                 **vars):
        apply(ZWiki.ZWikiPage.ZWikiPage.__init__,
              (self,source_string,mapping,__name__),vars)
        #apply(self.original.__init__,
        #      (self,source_string,mapping,__name__),vars)
        self.REQUEST = MockRequest()
        self._folder = OFS.Folder.Folder()
        self._folder.aq_parent = None # no use (see testCreateWithFileUpload)
        self.aq_parent = self._folder
        self.aq_parent.__class__.manage_addFolder = OFS.Folder.manage_addFolder
        self.aq_parent._setObject(self.getId(),self,set_owner=0)
        self.aq_inner = self
        self.aq_base = self

    def wikipage(self,body=None): return ''

    def getPhysicalPath(self): return ('',)

    def folder(self): return self._folder

    def _checkPermission(self, permission, object):
        return 1

    ZopeTime = DateTime.DateTime

    def cb_isMoveable(self):
        return 1

#Products.ZWiki.ZWikiPage.default_wikipage.getPhysicalRoot = lambda x:None

# OMG! the hackery 8-)
# allows handleFileUpload to work
#if not hasattr(MockZWikiPage,'original'):
#    MockZWikiPage.original = ZWiki.ZWikiPage.ZWikiPage
#    ZWiki.ZWikiPage.ZWikiPage = MockZWikiPage 

