"""
doctest support

Doctests are represented as python snippets within docstrings, and
appear throughout the zwiki code.  See doctest.py for more info.
Each of the tests below invokes doctest on a module and passes if all
the doctests pass.

notes:

- we need to use a hacked version of doctest to see the ZWikiPage
class, because it's based on an extension class.

- exceptions in doctests don't work because pyunit catches them first

- when reporting the number of tests, doctest counts contiguous
non-blank >>> lines as one test. 

- we sometimes run doctests twice, an undesirable slowdown

- get doctest to print line numbers in such a way that emacs can jump there

"""

from support import *
#import doctest_zope as doctest # python2ism
import doctest_zope; doctest = doctest_zope
from cStringIO import StringIO


class DocTests(unittest.TestCase):

    def _runDoctest(self, module, verbose=0, report=0):
        """
        helper method to invoke doctest on a module from within a
        pyunit test
        """
        doctest.master = None   # to get rid of merge warnings
        saveout=sys.stdout
        sys.stdout=StringIO()  # XXX some streaming here perhaps
        failures, tests = doctest.testmod(
            module,
            globs={#'ZWikiPageInDummyFolder':ZWikiPageInDummyFolder,
                   #'ZWiki':self.ZopeContext.manage_addProduct['ZWiki'],
            }, 
            verbose=verbose,
            report=report,
            isprivate=lambda x,y: 0 # test private things too
            )
        sys.stdout.flush()
        output = sys.stdout.getvalue()
        sys.stdout=saveout
        assert failures == 0, 'module %s (%d doctests):\n%s' % \
               (module.__name__,tests,output)
        return

    # run any doctests in the main modules
    # should scan for these automatically
    #XXX update this list!

    # runs ZWikiPage.py's doctests a second time ?
#    def testInitDoctests(self):
#        "doctests in __init.py__"
#        self._runDoctest(ZWiki.__init__)

    # XXX fails on zwiki.org
#    def testCatalogAwarenessDoctests(self):
#        "doctests in CatalogAwareness.py"
#        self._runDoctest(ZWiki.CatalogAwareness)

    def testDefaultsDoctests(self):
        "doctests in Defaults.py"
        self._runDoctest(ZWiki.Defaults)

    def testDiffDoctests(self):
        "doctests in Diff.py"
        self._runDoctest(ZWiki.Diff)

    def testParentsDoctests(self):
        "doctests in Parents.py"
        self._runDoctest(ZWiki.Parents)

    def testPermissionsDoctests(self):
        "doctests in Permissions.py"
        self._runDoctest(ZWiki.Permissions)

    def testRegexpsDoctests(self):
        "doctests in Regexps.py"
        self._runDoctest(ZWiki.Regexps)

    def testMailSupportDoctests(self):
        "doctests in Mail.py"
        self._runDoctest(ZWiki.Mail)

    def testZWikiPageDoctests(self):
        "doctests in ZWikiPage.py"
        self._runDoctest(ZWiki.ZWikiPage)

    def testZWikiWebDoctests(self):
        "doctests in ZWikiWeb.py"
        #ack this is a pain.. ZWikiWeb imports ZWikiPage causing (a) those
        #tests to be run twice and (b) trouble
        self._runDoctest(ZWiki.ZWikiWeb)

    def testWwmlDoctests(self):
        "doctests in wwml.py"
        self._runDoctest(ZWiki.wwml)

    #def testMailinDoctests(self):
    #    "doctests in Extensions/mailin.py"
    #    self._runDoctest(ZWiki.Extensions.mailin)


def test_suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(DocTests))
    return suite

def main():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    main()
